@@ -1,14 +1,13 @@
 '''
 stemdiff.dbase
 --------------
-Functions for the reading of 4D-STEM datafiles
-to create database of all files.
+Functions to calculate, save and re-read the database of 4D-STEM datafiles.
 
-* The database is a pandas DataFrame which contains the following
-  data for each datafile:
-  filename, XY-center, MaximumIntensity, NoOfPeaks, Entropy.
+* The database is saved as a pandas.DataFrame object,
+  which contains the following data for each datafile:
+  filename, XY-center coordinates, MaximumIntensity, NoOfPeaks, ShannonEntropy.
 * The database enables fast filtering of datafiles
-  and fast access to each datafile features,
+  and fast access to datafile features,
   which do not have to be re-calculated repeatedly.
 '''
 
@@ -19,15 +18,16 @@ from skimage import measure
     
 def calc_database(SDATA, DIFFIMAGES):
     """
-    Read 4D-STEM datafiles and calculate database of all files,
-    which contains [filename, S-entropy and XY-center] for each datafile.
+    Read 4D-STEM datafiles and calculate database of all files, which contains
+    [filename, XY-center coordinates, MaxIntensity, NoOfPeaks, ShannonEntropy]
+    for each datafile.
         
     Parameters
     ----------
     SDATA : stemdiff.gvars.SourceData object
         The object describes source data (detector, data_dir, filenames).
     DIFFIMAGES : stemdiff.gvars.DiffImages object
-        Object describing the diffraction images/patterns.
+        The object describing the diffraction images/patterns.
         
     Returns
     -------
@@ -36,33 +36,43 @@ def calc_database(SDATA, DIFFIMAGES):
         for each datafile in the dataset.
     """
     # Prepare variables before we run for-cycle to go through datafiles
-    # Initialize coordinates of the center = intensity center = primary beam
+    # (initialize coordinates of the center + rescale/upscale coefficient
     xc,yc = (None,None)
+    R = SDATA.detector.upscale
     # Initialize list of datafiles
     # (appending to lists is more efficient than appending to np/pd-structures
     list_of_datafiles = []
     # Pre-calculate additional variables
     # (it would be wasteful to calculate them in each for-cycle loop
-    # half_of_csquare = round(DIFFIMAGES.csquare/2)
     neighborhood_matrix = np.ones((DIFFIMAGES.peak_dist,DIFFIMAGES.peak_dist))
-    # Go through datafiles and calculated their entropy
+    # Go through datafiles and calculate their entropy
     for datafile in SDATA.filenames:
         # a) Read datafile
         arr = stemdiff.io.Datafiles.read(SDATA,datafile)
         # b) Get datafile name
         datafile_name = datafile.relative_to(SDATA.data_dir)
-        # c) Calculate center (of intensity)
+        # c) Calculate intensity center
+        # (to get high precision
+        # (the center must be determined for rescaled/upscaled array
         if DIFFIMAGES.ctype == 2:
             xc,yc = stemdiff.io.Arrays.find_center(
-                arr, DIFFIMAGES.csquare, DIFFIMAGES.cintensity)
+                stemdiff.io.Arrays.rescale(arr, R, order=3),  # rescaled array
+                DIFFIMAGES.csquare*R, DIFFIMAGES.cintensity)  # central region
         elif (DIFFIMAGES.ctype == 1) and (xc == None):
             xc,yc = stemdiff.io.Array.find_center(
-                arr, DIFFIMAGES.csquare, DIFFIMAGES.cintensity)
+                stemdiff.io.Arrays.rescale(arr, R, order=3),  # rescaled array
+                DIFFIMAGES.csquare*R, DIFFIMAGES.cintensity)  # central region
         elif (DIFFIMAGES.ctype == 0) and (xc == None):
-            geometric_center = round(SDATA.detector.detector_size/2)
+            geometric_center = round(SDATA.detector.detector_size*R/2)
             xc,yc = (geometric_center,geometric_center)
         # d) Determine maximum intensity
-        max_intensity = np.max(arr)
+        # NOTE: max_intensity must be saved as float
+        # Reason: when we later convert to pandas.Dataframe
+        # ...the datatypes are inferred from the first value in each column
+        # ...which may not be correct if the first integer is a small number
+        # ...as the estimated datatype may be int16
+        # ...and this may not work for larger values in the next datafiles
+        max_intensity = float(np.max(arr))
         # e) Estimate number of peaks (local maxima)
         no_of_maxima = stemdiff.io.Arrays.number_of_peaks(arr,
             peak_height = DIFFIMAGES.peak_height, 
@@ -77,19 +87,20 @@ def calc_database(SDATA, DIFFIMAGES):
     df = pd.DataFrame(
         list_of_datafiles,
         columns=['DatafileName','Xcenter','Ycenter', 'MaxInt', 'Peaks','S'])
+    
     # Return the dataframe containing names of datafiles + their entropies
     return(df)
 
 def save_database(df, output_file):
     """
-    Save database, which contains [filenames, entropies and XY-centers]
-    of all 4D-STEM datafiles; the dbase is saved as pickled object/zip-file.
+    Save the database of 4D-STEM datafiles;
+    the dbase is a pandas.DataFrame object saved as pickled object/zip-file.
         
     Parameters
     ----------
-    df : pandas DataFrame object
-        This object is a database of all datafiles,
-        which contains [filenames, entropies and XY-centers]
+    df : pandas.DataFrame object
+        This object is a database of all 4D-STEM datafiles,
+        which contains basic characteristics (filename, XY-center, entropy ...)
         of each datafile in 4D-STEM dataset.
     output_file : str
         Filename of the output file (without extension).
@@ -104,8 +115,7 @@ def save_database(df, output_file):
     
 def read_database(input_database):
     """
-    Read database, which contains [filenames, entropies and centers]
-    of all 4D-STEM datafiles.
+    Read the calculated database of 4D-STEM datafiles.
     
     Parameters
     ----------
@@ -114,11 +124,12 @@ def read_database(input_database):
         * pandas.Dataframe = dataframe that contains the database.
         * Why two possile types of input?
           If the database is in memory in the form of pandas.DataFrame
-          (which is quite common), it is useles to re-read it from file.
+          (which is quite common when using STEMDIFF package),
+          it is useles to re-read it from file.
           
     Returns
     -------
-    df : pandas DataFrame object
+    df : pandas.DataFrame object
         Database that has been read from disk or pandas.Dataframe.
     """
     # Read database from [input_file]
